/*
 * fake file functions for ramdisk use
 * Copyright (c) 1997 VM Labs, Inc.
 * All rights reserved.
 * Confidential and Proprietary Information
 * of VM Labs, Inc.
 *
 * This file, together with the files
 * in "ramdisk.s" and "filedata.s", implements
 * a simple read only ramdisk for Merlin
 * native apps. Reads and writes are
 * "faked" by doing copies from
 * memory.
 */

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "fileio.h"
#include "doomdef.h"

/*
 * each file on the ramdisk gets a
 * filedata structure in filedata.s
 */

#ifndef LOADFROMDISK
extern struct filedata {
    char *name;
    unsigned char *data;
    unsigned char *maxpos;
} predef_files[];
#else
typedef struct filedata {
    char *name;
    unsigned char *data;
    unsigned char *maxpos;
} FILEDATA;

FILEDATA predef_files[]=
{
{"doom1.wad",WAD_ADDRESS,WAD_SIZE},
{0,0,0}
};

#endif


/*
 * re-implementation of the low level
 * open, close, read, lseek functions;
 * plus a new function, MapFile, which
 * can be used for direct access to
 * the stored data in the ramdisk
 * (it returns a pointer to the data
 * stored in RAM -- this saves doing
 * the copy in read())
 */

struct file fileList[MAX_FILES];

int
open(const char *path, int access, ...) {
   va_list vp;
   int i;
   struct filedata *pf;

   vp = va_start(vp, access);

   /* first, look for the indicated name */
   for(pf = predef_files; pf->name; pf++) {
       if (!strcmp(path, pf->name))
	   break;
   }
   if (!pf->name) {
       errno = ENOENT;
       return -1;
   }
   for(i = 0; i < MAX_FILES; i++){
      if (!fileList[i].name) {
         fileList[i].name = strdup(path);
         fileList[i].pos = 0;
         fileList[i].access = access;
         if (access & 0x200) {   /* O_CREAT */
            fileList[i].mode = va_arg(vp, int);
         }
	 fileList[i].data = pf->data;
	 fileList[i].maxpos = (int)pf->maxpos;
         va_end(ap);
         return i;
      }
   }
   errno = EMFILE;
   return -1;
}

int close(int handle){
   if (!fileList[handle].name) {
      errno = EBADF;
      return -1;
   }
   free(fileList[handle].name);
   fileList[handle].name = 0;
   return 0;
}

int
read(int fd, void *ptr, unsigned long size)
{
    int bytesread;
    int maxpos;
    int pos;
    unsigned char *where = (unsigned char *)ptr;
    unsigned char *data;

    if (!fileList[fd].name) {
	errno = EBADF;
	return -1;
    }
    bytesread = 0;
    pos = fileList[fd].pos;
    maxpos = fileList[fd].maxpos;
    data = fileList[fd].data;

    while (size > 0 && pos < maxpos) {
	*where++ = data[pos++];
	bytesread++;
	--size;
    }

    fileList[fd].pos = pos;
    return bytesread;
}

#if 0
int
write(int fd, const void *ptr, unsigned long size)
{
    return size;
}
#endif

off_t
lseek(int fd, off_t offset, int whence)
{
    long newpos;
    long oldpos;
    long maxpos;

    if (!fileList[fd].name) {
	errno = EBADF;
	return -1;
    }
    oldpos = fileList[fd].pos;
    maxpos = fileList[fd].maxpos;

    if (whence == SEEK_SET) {
	newpos = offset;
    } else if (whence == SEEK_CUR) {
	newpos = oldpos + offset;
    } else if (whence == SEEK_END) {
	newpos = maxpos + offset;
    } else {
	errno = EINVAL;
	return -1;
    }
    if (newpos < 0)
	newpos = 0;
    else if (newpos > maxpos)
	newpos = maxpos;

    fileList[fd].pos = newpos;
    return newpos;
}

void *
MapFile(char *name, long *filesize)
{
   struct filedata *pf;

   /* first, look for the indicated name */
   for(pf = predef_files; pf->name; pf++) {
       if (!strcmp(name, pf->name))
	   break;
   }
   if (!pf) {
       *filesize = 0;
       return 0;
   }

   *filesize = (long)pf->maxpos;
   return pf->data;
}
